////////////////////////////////////////////////////////////////////////////
///
///   Locker object. It can protect access by using Lock(); and Release();
///   methods.
///
///   Example:
///
///     CLocker MyLocker;
///
///     MyLocker.Lock();
///     <read/write some data>
///     MyLocker.Release();
///
////////////////////////////////////////////////////////////////////////////

class CLocker
{
public:
  CLocker () { InitializeCriticalSection (& _critSection); }
  ~CLocker () { DeleteCriticalSection (& _critSection); }

public:
  void Lock ()
  {
    EnterCriticalSection (& _critSection);
  }
  BOOL TryLock ()
  {
    return TryEnterCriticalSection (& _critSection);
  }
  void Release ()
  {
    LeaveCriticalSection (& _critSection);
  }

private:
  CRITICAL_SECTION _critSection;
};

////////////////////////////////////////////////////////////////////////////
///
///   Pen and brush objects. Simplifies work with GDI drawing
///
////////////////////////////////////////////////////////////////////////////

class SPen
{
public:
    SPen (HDC hdc, COLORREF color)
        : _hdc (hdc)
    {
        _hPen = ::CreatePen (PS_SOLID, 0, color);
        _hPenOld = (HPEN)::SelectObject (_hdc, _hPen);
    }
    ~SPen ()
    {
        ::SelectObject (_hdc, _hPenOld);
        ::DeleteObject (_hPen);
    }
private:
    HDC     _hdc;
    HPEN    _hPen;
    HPEN    _hPenOld;
};

class SBrush
{
public:
    SBrush (HDC hdc, COLORREF color)
        : _hdc (hdc)
    {
        _hBrush = ::CreateSolidBrush (color);
        _hBrushOld = (HBRUSH)::SelectObject (_hdc, _hBrush);
    }
    ~SBrush ()
    {
        ::SelectObject (_hdc, _hBrushOld);
        ::DeleteObject (_hBrush);
    }
    operator HBRUSH () { return _hBrush; }
private:
    HDC     _hdc;
    HBRUSH  _hBrush;
    HBRUSH  _hBrushOld;
};