#include "stdafx.h"

void CCinemaProgress::DrawItemProgress(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
  DECLARE_MODULE_NAME;
  HDC     memDC;
  HBITMAP memBM;
  HGDIOBJ old_obj;

  memDC = ::CreateCompatibleDC ( lpDrawItemStruct->hDC );
  memBM = ::CreateCompatibleBitmap ( lpDrawItemStruct->hDC, lpDrawItemStruct->rcItem.right, lpDrawItemStruct->rcItem.bottom );
  old_obj = ::SelectObject ( memDC, memBM );

  if( !_enabled ) {
    /*
    // Clear Progress Bar
    HBRUSH brush,old_brush;
    brush     = ::CreateSolidBrush(SCR_COLOR_BLACK);
    old_brush = (HBRUSH)::SelectObject(memDC,brush);
    if( !::FillRect(memDC,&lpDrawItemStruct->rcItem,brush) ) {
      UtilsR1_ShowLastErrorMessage((char*)module);
    }
    ::SelectObject(memDC,old_brush);
    ::DeleteObject(brush);
    */
    // Clear Progress Bar
    SBrush brush(memDC,SCR_COLOR_BLACK);
    if( !::FillRect(memDC,&lpDrawItemStruct->rcItem,HBRUSH(brush)) ) {
//    UtilsR1_ShowLastErrorMessage((char*)module);
      UtilsR1_ShowLastErrorMessage("CCinemaProgress::DrawItemProgress: No 1");
    }
  }
  else {
    // Draw Progress Bar
    switch(_mode) {
    case CINEMAPROGRESS_TYPE_B:
      draw_item_progress_b(memDC,lpDrawItemStruct->rcItem);
      break;
    case CINEMAPROGRESS_TYPE_M:
      draw_item_progress_m(memDC,lpDrawItemStruct->rcItem);
      break;
    case CINEMAPROGRESS_TYPE_USER:
      if(_p_user_draw_function) _p_user_draw_function(memDC,lpDrawItemStruct->rcItem);
      break;
    }
  }

  // Drawing done, copy to screen
  ::BitBlt(lpDrawItemStruct->hDC, 0, 0, lpDrawItemStruct->rcItem.right, lpDrawItemStruct->rcItem.bottom, memDC, 0, 0, SRCCOPY);

  ::SelectObject ( memDC, old_obj );
  ::DeleteObject(memBM);
  ::DeleteDC(memDC);
}

void CCinemaProgress::draw_item_progress_b(HDC hDC,RECT &draw_rc)
{
  draw_item_progress(hDC,draw_rc,0);
}

void CCinemaProgress::draw_item_progress_m(HDC hDC,RECT &draw_rc)
{
  draw_item_progress(hDC,draw_rc,_size_window);
}

void CCinemaProgress::draw_item_progress(HDC hDC,RECT &draw_rc,int marker_width)
{
  DECLARE_MODULE_NAME;
  LONG   x0,y0,x1,y1;
  POINT  pt[5];
  /*
  HPEN   pen,old_pen;
  HBRUSH brush,old_brush;
  */

  //// Big rectangle //////////////////////////////////////////////////////////////////
  {
    // Coords
    x0 = 0;
    y0 = draw_rc.top+2;
    x1 = draw_rc.right-1;
    y1 = draw_rc.bottom-3;

    // Drawing tools
    /*
    pen     = ::CreatePen(PS_SOLID,1,SCR_COLOR_GRAY);
    old_pen = (HPEN)::SelectObject(hDC,pen);
    */
    SPen GrayPen(hDC,SCR_COLOR_GRAY);

    // Draw rectangle
    pt[0].x = x0; pt[0].y = y0;
    pt[1].x = x1; pt[1].y = y0;
    pt[2].x = x1; pt[2].y = y1;
    pt[3].x = x0; pt[3].y = y1;
    pt[4].x = x0; pt[4].y = y0;
    if( !::Polyline(hDC,pt,5) ) {
//    UtilsR1_ShowLastErrorMessage((char*)module);
      UtilsR1_ShowLastErrorMessage("CCinemaProgress::DrawItemProgress: No 2");
    }
  }
  //// Small rectangle ////////////////////////////////////////////////////////////////
  double pix_per_point = ((double)draw_rc.right) / ((double)_size_total);
  {
    // Coords
    x0 = x0+1;
    y0 = y0+1;
    x1 = f2i((double)_size_used*pix_per_point);
    if(x1 >= (draw_rc.right-1)) x1 = draw_rc.right-2;
    y1 = y1-1;

    // Drawing tools
    /*
    pen = ::CreatePen(PS_SOLID,1,SCR_COLOR_LIGHT_GREEN);
    ::SelectObject(hDC,pen);
    brush     = ::CreateSolidBrush(SCR_COLOR_LIGHT_GREEN);
    old_brush = (HBRUSH)::SelectObject(hDC,brush);
    */
    SPen   pen(hDC,SCR_COLOR_LIGHT_GREEN);
    SBrush brush(hDC,SCR_COLOR_LIGHT_GREEN);

    // Draw polygon
    pt[0].x = x0; pt[0].y = y0;
    pt[1].x = x1; pt[1].y = y0;
    pt[2].x = x1; pt[2].y = y1;
    pt[3].x = x0; pt[3].y = y1;
    pt[4].x = x0; pt[4].y = y0;
    if( !::Polygon(hDC,pt,5) ) {
//    UtilsR1_ShowLastErrorMessage((char*)module);
      UtilsR1_ShowLastErrorMessage("CCinemaProgress::DrawItemProgress: No 3");
    }
  }
  //// Position marker ////////////////////////////////////////////////////////////////
  {
    // Coords
    int marker_w = f2i((double)marker_width*pix_per_point);
    if(marker_w <= 0) marker_w = 1;
    x0 = f2i((double)_position*pix_per_point)+1;
    y0 = draw_rc.top;
    x1 = x0+marker_w;
    y1 = draw_rc.bottom-1;

    // Drawing tools
    /*
    pen = ::CreatePen(PS_SOLID,1,SCR_COLOR_YELLOW);
    ::SelectObject(hDC,pen);
    */
    SPen pen(hDC,SCR_COLOR_YELLOW);

    // Draw rectangle
    pt[0].x = x0; pt[0].y = y0;
    pt[1].x = x1; pt[1].y = y0;
    pt[2].x = x1; pt[2].y = y1;
    pt[3].x = x0; pt[3].y = y1;
    pt[4].x = x0; pt[4].y = y0;
    if( !::Polyline(hDC,pt,5) ) {
//    UtilsR1_ShowLastErrorMessage((char*)module);
      UtilsR1_ShowLastErrorMessage("CCinemaProgress::DrawItemProgress: No 4");
    }
  }
  /*
  // Clean up
  ::SelectObject(hDC,old_pen);
  ::SelectObject(hDC,old_brush);
  */
}
